package com.sksamuel.scoot.rest

import javax.ws.rs.core.MediaType
import com.sksamuel.soa.rest.RestClient

/** @author Stephen Samuel
  *
  *         A base implementation of a client that connects to a CRUD Rest Endpoint
  *
  * */
abstract class CrudRestClient[T: Manifest, ID] {

    var client = new RestClient

    /**
     * Override this method to return the base path of the resource.
     *
     * @return the resource path
     */
    def resourceUrl: String

    def delete(id: ID): Int = client.resource(resourceUrl + "/" + id).delete.status

    def save(t: T): RemoteResponse[T] = client.resource(resourceUrl).post(t, MediaType.APPLICATION_JSON_TYPE).invoke[T]

    /**
     * Returns the resource for the given id
     *
     */
    def get(id: String): RemoteResponse[T] = client.resource(resourceUrl + "/" + id).get.invoke[T]

    /**
     * Returns a list of all the resources
     */
    def list: RemoteResponse[Array[T]] = client.resource(resourceUrl).get.invoke[Array[T]]
}
