package com.sksamuel.scoot.rest

import java.io.{ByteArrayOutputStream, InputStream}
import org.codehaus.jackson.map.ObjectMapper
import com.fasterxml.jackson.module.scala.DefaultScalaModule
import javax.xml.bind.JAXBContext

/** @author Stephen Samuel */
abstract class Marshaller {
    def marshall(any: Any): Array[Byte]
    def unmarshall[T](input: InputStream)(implicit m: Manifest[T]): T
}

object JsonMarshaller extends Marshaller {

    private val mapper = new ObjectMapper
    mapper.registerModule(DefaultScalaModule)

    def marshall(any: Any): Array[Byte] = mapper.writeValueAsBytes(any)
    def unmarshall[T](input: InputStream)(implicit m: Manifest[T]): T = mapper
      .readValue(input, m.erasure.asInstanceOf[Class[T]])
}

object XmlMarshaller extends Marshaller {

    def marshall(any: Any): Array[Byte] = {
        val ba = new ByteArrayOutputStream()
        JAXBContext.newInstance(any.getClass).createMarshaller().marshal(any, ba)
        ba.toByteArray
    }

    def unmarshall[T](input: InputStream)(implicit m: Manifest[T]): T =
        JAXBContext.newInstance(m.erasure).createUnmarshaller().unmarshal(input).asInstanceOf[T]

}