package com.sksamuel.scoot.rest

abstract class RemoteResponse[+T](status: Int) {
    def success: Boolean
    final def error: Boolean = !success
    def option: Option[T]
    final def getOrElse[B >: T](e: => B): B = option.getOrElse(e)
    final def orNull[A1 >: T](implicit ev: Null <:< A1): A1 = option.getOrElse(null)
    final def orException(m: String) = option.getOrElse(throw new RuntimeException(m))
    final def foreach[U](f: T => U) {
        option.foreach(f)
    }
    final def map[B](f: T => B) = option.map(f)
}
final case class Ok[T](t: T) extends RemoteResponse[T](200) {
    def success = true
    def option = Some(t)
}
final case class NoContent(status: Int) extends RemoteResponse[Nothing](200) {
    def success = true
    def option = None
}
final case class ResponseError(status: Int, message: String) extends RemoteResponse[Nothing](status) {
    def success = false
    def option = None
}

object RemoteResponse {
    // copy constructor
    def apply[T](resp: RemoteResponse[T]): RemoteResponse[T] = {
        resp match {
            case Ok(t) => Ok(t)
            case NoContent(s) => NoContent(204)
            case ResponseError(s, m) => ResponseError(s, m)
        }
    }
    def nocontent = new NoContent(204)
    def ok[T](value: T) = new Ok[T](value)
    def error(status: Int, message: String) = new ResponseError(status, message)
}
